#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "set.h"
#include "scan_word.h"

void lowercase(stralloc* s) {
   for (int index = 0; index < s->len; ++index) {
      char ch = s->s[index];
      if (isupper(ch)) {
	 ch = tolower(ch);
	 s->s[index] = ch;
      }
   }
}

int main(int argc, char** argv) {
   char* cmdname = *argv;
   char usage[] = "Usage: %s [-i] [-v] dict\n";
   bool i_flag = false;
   bool v_flag = false;

   int option;
   while ((option = getopt(argc, argv, "iv")) != -1) {
      switch (option) {
	 case 'i':
	    i_flag = true;
	    break;
	 case 'v':
	    v_flag = true;
	    break;
	 default:
	    fprintf(stderr, usage, cmdname); exit(1);
	    break;
      }
   }
   argv += optind; argc -= optind;
   if (argc != 1) {
      fprintf(stderr, usage, cmdname); exit(1);
   }
   char* dict_file = argv[0];
   Set* dict = create_set(8192);
   Set* words = create_set(8192);
   if (!dict || !words) {
      fprintf(stderr, "%s: out of memory\n", cmdname); exit(1);
   }

   /* read dictionary */
   FILE* fp = fopen(dict_file, "r");
   if (!fp) {
      perror(dict_file); exit(1);
   }
   stralloc word = {0};
   while (scan_word(fp, &word)) {
      if (i_flag) lowercase(&word);
      set_add(dict, &word);
   }
   fclose(fp);

   /* process stdin */
   while (scan_word(stdin, &word)) {
      if (i_flag) lowercase(&word);
      bool include = in_set(dict, &word);
      if (v_flag) include = !include;
      if (include) set_add(words, &word);
   }

   /* print words */
   set_first(words);
   const stralloc* w;
   while ((w = set_next(words))) {
      printf("%.*s\n", (int)w->len, w->s);
   }
}
